(function () {
	var dashboardInterval = 10000;
	var now = new Date().getTime();
	var cpu = [], ramMemory = [], swapMemory = [], loadAverage = [];
	var activeCalls = [], activeChannels = [];

	/* Plot Settings */
	var plotSeries = {
		lines:{
			show: true,
			fill: .3,
			lineWidth: 1,
			fillColor:{
				colors: [ { opacity: 0.2 },{ opacity: 0.9 } ]
			}
		},
		points: {
			show: false,
			lineWidth: 1.5,
			radius: 2,
			symbol: "circle",
			fill: true,
			fillColor: "#ffffff"
		},
		shadowSize: 0,
		curvedLines: {
			active: true,
			apply: true,
			fit: true,
			fitPointDist: 0.000001
		}
	};

	var plotGrid = {
		hoverable: true,
		clickable: true,
		borderColor: "#fff",
		borderWidth: 0,
		labelMargin: 10,
		backgroundColor: "#fff"
	};

	var plotXAxis = {
		mode: "time",
		color: 'rgba(0,0,0,0)',
		tickSize: [60, "second"],
		tickFormatter: function (v, axis) {
			var date = new Date(v);

			if (date.getSeconds() % 20 == 0) {
				var hours = date.getHours() < 10 ? "0" + date.getHours() : date.getHours();
				var minutes = date.getMinutes() < 10 ? "0" + date.getMinutes() : date.getMinutes();
				var seconds = date.getSeconds() < 10 ? "0" + date.getSeconds() : date.getSeconds();

				return hours + ":" + minutes;
			} else {
				return "";
			}
		},
		axisLabel: "Time"
	};

	initChartsData();
	hardDiskPie();

	$("#tab_dashboard").setInterval(getDashboardInfo, dashboardInterval, true);

	function getDashboardInfo(){
		var tab = $('ul#ombutel-main-tabs > li').find('a[aria-controls="tab_dashboard"]');
		//Request to Server
		$.ajax({
			dataType: 'json',
			data: { class: 'dashboard', method: 'getDashboardData' },
			success: function (Response) {
				updateUptime(
					Response.current_time,
					Response.asterisk.asterisk_uptime,
					Response.server_uptime
				);
				if($('#cpu-placeholder').length > 0){
					updateCPUChart(Response.cpu_info);
				}

				if($('#asterisk-call-placeholder').length > 0){
					updateAsteriskChart(Response.asterisk);
				}
				updateServicesStatus(Response.services_status);
				Response.networks.time = Date.now();
				updateNetworkUsage(Response.networks);
			}
		});
	}

	function updateUptime(current_time, asteriskUptime, serverUptime) {
		$('span.span-current_time').text(current_time);
		$('span.span-asterisk_uptime').text(asteriskUptime);
		$('span.span-server_uptime').text(serverUptime);
	}

	function hardDiskPie(){
		$.plot("#hdd-placeholder", HardDiskDataSet, {
			series: {
				 pie: {
					show: true,
					combine: {
						color: "#999",
						threshold: 0.05
					}
				}
			},
			legend: {
				show: false
			},
			 grid: {
				/*hoverable: true*/
			}
		});
	}

	function updateServicesStatus(data) {
		var icons = {"active": "fa-check-circle", "inactive": "fa-times-circle", "unknown": "fa-exclamation-circle"};
		$.each(data, function (i,value) {
			var span = $('.span-' + i);
			var icon = $('.icon-' + i);
			var serviceClass = 'dash-service-' + value;

			icon.removeClass('fa-check-circle fa-times-circle fa-exclamation-circle dash-service-active dash-service-stopped dash-service-unknown');
			icon.addClass(icons[value] + " " + serviceClass);
			span.text(value);
		});
	}

	function human_readable(value, suffix) {
		var levels = [ '', 'Ki', 'Mi', 'Gi', 'TiB' ];
		var level = Math.max(Math.floor((Math.log(value) / Math.LN10) / 3), 0);
		return Math.floor(value / Math.pow(1000, level)) + ' ' + levels[level] + suffix;
	}

	function updateNetworkUsage(data) {
		var networkUL = $("ul#dash-network-items");
		networkUL.html('');
		var now = Date.now();
		var previous = this.previous;
		$.each(data, function(interface, current) {
			if (interface != 'time') {
				var rx = 0;
				var tx = 0;
				if (previous && previous[interface]) {
					rx = ((current.rx - previous[interface].rx) / (now - previous.time)) * 1000;
					tx = ((current.tx - previous[interface].tx) / (now - previous.time)) * 1000;
				}
				networkUL.append(networkTemplate(interface, human_readable(rx, 'B/s'), human_readable(tx, 'B/s')));
			}
		});
		this.previous = data;
	}

	function networkTemplate(netName, transmited, recived){
		var template = '<li> ' +
			'<div class="icon-section"> <i class="fa fa-cloud fa-fw"></i> </div>' +
			'<div class="content-section"> ' +
			'<span class="dash-label"> '+ netName +' </span> ' +
			'<span class="desc"> tx: '+ transmited +', rx: '+ recived +' </span> ' +
			'</div>' +
			' </li>';

		return template;
	}

	function initChartsData(){
		for (var i = 0; i < 20; i++) {
			temp = [now += dashboardInterval, 0];

			cpu.push(temp);
			ramMemory.push(temp);
			swapMemory.push(temp);
			loadAverage.push(temp);
			activeCalls.push(temp);
			activeChannels.push(temp);
		}
	}

	function updateCPUChart(cpuInfo){
		cpu.shift();
		ramMemory.shift();
		swapMemory.shift();
		loadAverage.shift();
		var temp;
		now += dashboardInterval;

		temp = [now, cpuInfo.cpu_utilization];
		cpu.push(temp);

		temp = [now, cpuInfo.mem_usage];
		ramMemory.push(temp);

		temp = [now, cpuInfo.swap_memory];
		swapMemory.push(temp);

		temp = [now, cpuInfo.load_avg];
		loadAverage.push(temp);

		var dataSet = [
			{ label: "CPU", data: cpu, color: "#3498db" },
			{ label: "Memory", data: ramMemory, color: "#e67e22" },
			{ label: "Swap", data: swapMemory, color: "#e74c3c" },
			{ label: "Load Average", data: loadAverage, color: "#8e44ad" }
		];

		$.plot($('#cpu-placeholder'),dataSet,{
			series: plotSeries,
			legend: {
				position: "nw"
			},
			grid: plotGrid,
			tooltip: true,
			xaxis: plotXAxis,
			yaxis:{
				min: 0,
				max: 100,
				tickSize: 5,
				tickFormatter: function (v, axis) {
					if (v % 10 == 0) {
						return v + "%";
					} else {
						return "";
					}
				},
				color: 'rgba(0,0,0,0)',
				axisLabel: "Percentage",
				axisLabelUseCanvas: true,
				axisLabelFontSizePixels: 12,
				axisLabelFontFamily: 'Verdana, Arial',
				axisLabelPadding: 6
			}
		});

		$('#cpu-placeholder').bind("plothover", function (event, pos, item){
			if(item){
				var x = item.datapoint[0],
					y = item.datapoint[1].toFixed(2);

				$("#flotTip").remove();
				showPlotTooltip(item.pageX, item.pageY, item.series.label +': ' + y + '%');
			}else{
				$("#flotTip").remove();
			}
		});
	}

	function updateAsteriskChart(asteriskInfo){
		activeCalls.shift();
		activeChannels.shift();
		var temp;
		now += dashboardInterval;

		temp = [now, asteriskInfo.active_calls];
		activeCalls.push(temp);

		temp = [now, asteriskInfo.active_channels];
		activeChannels.push(temp);

		var dataSet = [
			{ label: "Active Calls", data: activeCalls, color: "#00BCD4" },
			{ label: "Active Channels", data: activeChannels, color: "#795548" }
		];

		$.plot($('#asterisk-call-placeholder'),dataSet,{
			series: plotSeries,
			legend: {
				position: "nw"
			},
			grid: plotGrid,
			tooltip: true,
			xaxis: plotXAxis,
			yaxis:{
				min: 0,
				tickDecimals:0,
				color: 'rgba(0,0,0,0)',
				axisLabel: "Calls",
				axisLabelUseCanvas: true,
				axisLabelFontSizePixels: 12,
				axisLabelFontFamily: 'Verdana, Arial',
				axisLabelPadding: 6
			}
		});

		$('#asterisk-call-placeholder').bind("plothover", function (event, pos, item){
			if(item){
				var x = item.datapoint[0],
					y = item.datapoint[1].toFixed(0);

				$("#flotTip").remove();
				showPlotTooltip(item.pageX, item.pageY, item.series.label +': ' + y);
			}else{
				$("#flotTip").remove();
			}
		});
	}

	function showPlotTooltip(x, y, contents) {
		$('<div id="flotTip">' + contents + '</div>').css( {
			position: "absolute",
			top: y + 5,
			left: x + 5
		}).appendTo("body").fadeIn(200);
	}
})();
