(function() {
	var session, userAgent;
	var onCall = false, timerRinging, noRinging = false;

	//Register Function
	$('#register').click(function () {
		userAgent = new SIP.UA({
			traceSip: false,
			userAgentString: 'Ombutel/webRTC v 1.0',
			uri: '',
			wsServers: '',
			authorizationUser: '',
			password: '',
			displayName: ''
		});

		registerEvents(userAgent);
	});

	//Unregister Function
	$('#unregister').click(function () {
		userAgent.unregister();
	});

	//MakeCall Function
	$('#call').click(function () {
		makeCall(userAgent, session);
	});

	//Phone Events
	$('.keys span').click(function() {
		var btnVal = $(this).text();
		var actualVal = $('#target').text();

		$('#keys').trigger('play');
		$('#target').text(actualVal+btnVal);
	});

	$('#erase').click(function() {
		var actualVal = $('#target').text();
		$('#target').text(actualVal.substring(0, actualVal.length-1));
	});

	//Register the events to UserAgent
	function registerEvents(userAgent) {
		userAgent.on('connecting', function () {
			$('#status').text('Connecting');
		});

		userAgent.on('connected', function () {
			$('#status').text('Connected');
		});

		userAgent.on('disconnected', function () {
			$('#status').text('Disconnected');
		});

		userAgent.on('registered', function () {
			$('#status').text('Registered');
		});

		userAgent.on('unregistered', function () {
			$('#status').text('Unregistered');
		});

		userAgent.on('invite', function (session) {
			incomingCall(session.remoteIdentity.uri, session);
		});
	}

	//Make a call
	function makeCall(userAgent, session) {
		var target = $('#target').text();

		var renderHint = {
			remote: {
				audio: document.getElementById('remote-audio')
			},
			local: {
				audio: document.getElementById('local-audio')
			}
		};

		session = userAgent.invite('sip:'+target+'@'+server, getOptions());

		$('.btn-call').addClass('disabled');
		$('.btn-config').addClass('disabled');
		$('.btn-endcall').removeClass('disabled');
		$('#loader').removeClass('hidden');
		$('#status').text('Connecting...');
		$('#in-call').text('You are calling to');

		session.on('progress', function () {
			startRingbacktone();
			$('#status').text('Ringing...');
		});

		session.on('accepted',function(){
			stopRingbacktone();
			session.mediaHandler.render(renderHint);

			$('.btn-call').addClass('disabled');
			$('#loader').addClass('hidden');
			$('#status').text('In Call Now');

			onCall = true;
		});

		//Add Streaming
		session.mediaHandler.on('addStream', function () {
			session.mediaHandler.render(renderHint);
		});

		sessionEvents(session);
	}

	//Receive a call
	function incomingCall(uri, session) {
		startRingtone();
		$('.btn-call').addClass('hidden');
		$('.btn-accept-call').removeClass('hidden');
		$('.btn-endcall').removeClass('disabled');

		$('#status').text('Incoming Call...');
		$('.btn-config').addClass('disabled');
		$('.btn-endcall').removeClass('disabled');

		var renderHint = {
			remote: {
				audio: document.getElementById('remote-audio')
			},
			local: {
				audio: document.getElementById('local-audio')
			}
		};

		if (onCall) {

		} else {
			timerRinging = setTimeout(function() {
				ringing();
			}, 2000);

			$('#in-call').text(session.remoteIdentity.uri);
			$('#target').text(session.remoteIdentity.displayName);

			$('#accept-call').click(function() {
				stopRingtone();
				$('#loader').removeClass('hidden');
				$('#status').text('Linking Call...');
				noRinging = true;

				session.accept(getOptions());
				$('.btn-accept-call').addClass('disabled');
			});

			session.on('accepted',function(){
				session.mediaHandler.render(renderHint);

				$('.btn-call').addClass('disabled');
				$('#loader').addClass('hidden');
				$('#status').text('In Call Now');

				onCall = true;
			});

			//Add Streaming
			session.mediaHandler.on('addStream', function () {
				session.mediaHandler.render(renderHint);
			});

			noRinging = false;
			sessionEvents(session);
		}
	}

	//Manage session events
	function sessionEvents(session) {
		session.on('bye', function () {
			defaultValues('Call Terminated');
			deleteSession('Bye Event', session);
		});

		session.on('rejected', function () {
			defaultValues('Call Not Accepted')
			deleteSession('Rejected Event', session);
			//session.terminate();
		});

		session.on('failed', function () {
			defaultValues('Call Failed')
			deleteSession('Failed Event', session);
		});

		session.on('cancel', function () {
			defaultValues('Call Canceled')
			deleteSession('Cancel Event', session);
		});

		session.on('terminated', function () {
			deleteSession('Terminated Event', session);
		});

		$('#end-call').click(function () {
			if (!session) {
				return;
			} else if (session.startTime) {
				session.bye();
			} else if (session.reject) {
				session.reject();
			} else if (session.cancel) {
				session.cancel();
			}

			deleteSession('End Call Button'), session;
		});

	}

	function ringing() {
		$("#accept-call").effect('shake', {distance:5});

		if (onCall == true || noRinging == true) {
			clearTimeout(timerRinging, 100);
		} else {
			setTimeout(ringing, 2000);
		}
	}

	//Media Options
	function getOptions(){
		var options = {
			media: {
				constraints: {
					audio: true,
					video: false
				}
			}
		};

		return options;
	}

	function startRingtone() {
		$('#ringtone').trigger('play');
	}

	function stopRingtone() {
		$('#ringtone').trigger('stop');
		$('#ringtone').removeAttr('loop');
	}

	function startRingbacktone() {
		$('#ringbacktone').trigger('play');
	}

	function stopRingbacktone() {
		$('#ringbacktone').removeAttr('loop');
		$('#ringbacktone').trigger('stop');
	}

	//Return the default Values
	function defaultValues(message) {
		//Hide Loader
		$('#loader').addClass('hidden');

		//Show Status
		$('#status').text(message);

		//Clean the screen
		$('.information-call p').text('');

		onCall = false;
		noRinging = true;

		$('.btn-config').removeClass('disabled');
		$('.btn-call').removeClass('disabled');
		$('.btn-accept-call').removeClass('disabled');
		$('.btn-endcall').addClass('disabled');

		$('.btn-call').removeClass('hidden');
		$('.btn-accept-call').addClass('hidden');
	}

	function deleteSession($fairedEvent, $session){
		stopRingbacktone();
		stopRingtone();
		alert('Fired at ' + $fairedEvent);

		if(!$session){
			alert('La session no existe');
		}else{
			alert('La session existe');
			$session.bye();
			session = null;
			delete session; //Delete Object
		}

		console.log('session data: ' + $session);
	}
})();
